@@ -3,13 +3,17 @@ package phase
 import (
 	"bufio"
 	"bytes"
+	"context"
 	"fmt"
+	"path/filepath"
 	"strings"
 	"time"
 
 	"github.com/k0sproject/dig"
 	"github.com/k0sproject/k0sctl/pkg/apis/k0sctl.k0sproject.io/v1beta1"
 	"github.com/k0sproject/k0sctl/pkg/apis/k0sctl.k0sproject.io/v1beta1/cluster"
+	"github.com/k0sproject/k0sctl/pkg/node"
+	"github.com/k0sproject/k0sctl/pkg/retry"
 	"github.com/k0sproject/rig/exec"
 	log "github.com/sirupsen/logrus"
 	"gopkg.in/yaml.v2"
@@ -62,13 +66,13 @@ func (p *ConfigureK0s) Run() error {
 	return p.parallelDo(controllers, p.configureK0s)
 }
 
-func (p *ConfigureK0s) validateConfig(h *cluster.Host) error {
+func (p *ConfigureK0s) validateConfig(h *cluster.Host, configPath string) error {
 	log.Infof("%s: validating configuration", h)
 	var cmd string
 	if h.Exec(h.Configurer.K0sCmdf("config validate --help"), exec.Sudo(h)) == nil {
-		cmd = h.Configurer.K0sCmdf(`config validate --config "%s"`, h.K0sConfigPath())
+		cmd = h.Configurer.K0sCmdf(`config validate --config "%s"`, configPath)
 	} else {
-		cmd = h.Configurer.K0sCmdf(`validate config --config "%s"`, h.K0sConfigPath())
+		cmd = h.Configurer.K0sCmdf(`validate config --config "%s"`, configPath)
 	}
 
 	output, err := h.ExecOutput(cmd, exec.Sudo(h))
@@ -104,26 +108,44 @@ func (p *ConfigureK0s) configureK0s(h *cluster.Host) error {
 		return err
 	}
 
-	if err := h.Configurer.WriteFile(h, h.K0sConfigPath(), cfg, "0600"); err != nil {
+	tempConfigPath, err := h.Configurer.TempFile(h)
+	if err != nil {
+		return fmt.Errorf("failed to create temporary file for config: %w", err)
+	}
+
+	if err := h.Configurer.WriteFile(h, tempConfigPath, cfg, "0600"); err != nil {
 		return err
 	}
 
-	if err := p.validateConfig(h); err != nil {
+	if err := p.validateConfig(h, tempConfigPath); err != nil {
 		return err
 	}
 
 	if equalConfig(oldcfg, cfg) {
 		log.Debugf("%s: configuration did not change", h)
 	} else {
-		log.Infof("%s: configuration was changed", h)
-		if h.Metadata.K0sRunningVersion != "" && !h.Metadata.NeedsUpgrade {
+		log.Infof("%s: configuration was changed, installing new configuration", h)
+		configPath := h.K0sConfigPath()
+		configDir := filepath.Dir(configPath)
+
+		if !h.Configurer.FileExist(h, configDir) {
+			if err := h.Execf(`install -d 0750 -o root -g root "%s"`, configDir, exec.Sudo(h)); err != nil {
+				return fmt.Errorf("failed to create k0s configuration directory: %w", err)
+			}
+		}
+
+		if err := h.Execf(`install -m 0600 -o root -g root "%s" "%s"`, tempConfigPath, configPath, exec.Sudo(h)); err != nil {
+			return fmt.Errorf("failed to install k0s configuration: %w", err)
+		}
+
+		if h.Metadata.K0sRunningVersion != nil && !h.Metadata.NeedsUpgrade {
 			log.Infof("%s: restarting the k0s service", h)
 			if err := h.Configurer.RestartService(h, h.K0sServiceName()); err != nil {
 				return err
 			}
 
 			log.Infof("%s: waiting for the k0s service to start", h)
-			return h.WaitK0sServiceRunning()
+			return retry.Timeout(context.TODO(), retry.DefaultTimeout, node.ServiceRunningFunc(h, h.K0sServiceName()))
 		}
 	}
 
@@ -162,7 +184,7 @@ func addUnlessExist(slice *[]string, s string) {
 func (p *ConfigureK0s) configFor(h *cluster.Host) (string, error) {
 	var cfg dig.Mapping
 	// Leader will get a full config on initialize only
-	if !p.Config.Spec.K0s.DynamicConfig || (h == p.leader && h.Metadata.K0sRunningVersion == "") {
+	if !p.Config.Spec.K0s.DynamicConfig || (h == p.leader && h.Metadata.K0sRunningVersion == nil) {
 		cfg = p.Config.Spec.K0s.Config.Dup()
 	} else {
 		cfg = p.Config.Spec.K0s.NodeConfig()
