@@ -5,7 +5,6 @@ import (
 
 	"github.com/k0sproject/k0sctl/pkg/apis/k0sctl.k0sproject.io/v1beta1"
 	"github.com/k0sproject/k0sctl/pkg/apis/k0sctl.k0sproject.io/v1beta1/cluster"
-	"github.com/k0sproject/version"
 	log "github.com/sirupsen/logrus"
 )
 
@@ -36,7 +35,7 @@ func (p *DownloadK0s) Prepare(config *v1beta1.Cluster) error {
 		}
 
 		// The version on host is already same as the target version
-		if p.Config.Spec.K0s.VersionEqual(h.Metadata.K0sBinaryVersion) {
+		if p.Config.Spec.K0s.Version.Equal(h.Metadata.K0sBinaryVersion) {
 			log.Debugf("%s: k0s version on target host is already %s", h, h.Metadata.K0sBinaryVersion)
 			return false
 		}
@@ -58,18 +57,13 @@ func (p *DownloadK0s) Run() error {
 }
 
 func (p *DownloadK0s) downloadK0s(h *cluster.Host) error {
-	targetVersion, err := version.NewVersion(p.Config.Spec.K0s.Version)
-	if err != nil {
-		return fmt.Errorf("parse k0s version: %w", err)
-	}
-
 	tmp, err := h.Configurer.TempFile(h)
 	if err != nil {
 		return fmt.Errorf("failed to create tempfile %w", err)
 	}
 
-	log.Infof("%s: downloading k0s %s", h, targetVersion)
-	if err := h.Configurer.DownloadK0s(h, tmp, targetVersion, h.Metadata.Arch); err != nil {
+	log.Infof("%s: downloading k0s %s", h, p.Config.Spec.K0s.Version)
+	if err := h.Configurer.DownloadK0s(h, tmp, p.Config.Spec.K0s.Version, h.Metadata.Arch); err != nil {
 		return err
 	}
 
