@@ -1,6 +1,7 @@
 package phase
 
 import (
+	"context"
 	"encoding/json"
 	"fmt"
 	"path"
@@ -8,6 +9,7 @@ import (
 
 	"github.com/k0sproject/dig"
 	"github.com/k0sproject/k0sctl/pkg/apis/k0sctl.k0sproject.io/v1beta1/cluster"
+	"github.com/k0sproject/k0sctl/pkg/node"
 	"github.com/k0sproject/rig/exec"
 	"github.com/k0sproject/version"
 	log "github.com/sirupsen/logrus"
@@ -15,16 +17,16 @@ import (
 )
 
 type k0sstatus struct {
-	Version       string      `json:"Version"`
-	Pid           int         `json:"Pid"`
-	PPid          int         `json:"PPid"`
-	Role          string      `json:"Role"`
-	SysInit       string      `json:"SysInit"`
-	StubFile      string      `json:"StubFile"`
-	Workloads     bool        `json:"Workloads"`
-	Args          []string    `json:"Args"`
-	ClusterConfig dig.Mapping `json:"ClusterConfig"`
-	K0sVars       dig.Mapping `json:"K0sVars"`
+	Version       *version.Version `json:"Version"`
+	Pid           int              `json:"Pid"`
+	PPid          int              `json:"PPid"`
+	Role          string           `json:"Role"`
+	SysInit       string           `json:"SysInit"`
+	StubFile      string           `json:"StubFile"`
+	Workloads     bool             `json:"Workloads"`
+	Args          []string         `json:"Args"`
+	ClusterConfig dig.Mapping      `json:"ClusterConfig"`
+	K0sVars       dig.Mapping      `json:"K0sVars"`
 }
 
 func (k *k0sstatus) isSingle() bool {
@@ -75,7 +77,12 @@ func (p *GatherK0sFacts) investigateK0s(h *cluster.Host) error {
 		return nil
 	}
 
-	h.Metadata.K0sBinaryVersion = strings.TrimSpace(output)
+	binVersion, err := version.NewVersion(strings.TrimSpace(output))
+	if err != nil {
+		return fmt.Errorf("failed to parse installed k0s version: %w", err)
+	}
+
+	h.Metadata.K0sBinaryVersion = binVersion
 
 	log.Debugf("%s: has k0s binary version %s", h, h.Metadata.K0sBinaryVersion)
 
@@ -124,7 +131,7 @@ func (p *GatherK0sFacts) investigateK0s(h *cluster.Host) error {
 		return nil
 	}
 
-	if status.Version == "" || status.Role == "" || status.Pid == 0 {
+	if status.Version == nil || status.Role == "" || status.Pid == 0 {
 		log.Debugf("%s: k0s is not running", h)
 		return nil
 	}
@@ -148,7 +155,7 @@ func (p *GatherK0sFacts) investigateK0s(h *cluster.Host) error {
 		return fmt.Errorf("%s: is configured as k0s %s but is already running as %s - role change is not supported", h, h.Role, status.Role)
 	}
 
-	h.Metadata.K0sRunningVersion = strings.TrimPrefix(status.Version, "v")
+	h.Metadata.K0sRunningVersion = status.Version
 	h.Metadata.NeedsUpgrade = p.needsUpgrade(h)
 
 	log.Infof("%s: is running k0s %s version %s", h, h.Role, h.Metadata.K0sRunningVersion)
@@ -185,11 +192,11 @@ func (p *GatherK0sFacts) investigateK0s(h *cluster.Host) error {
 
 	if !h.IsController() {
 		log.Infof("%s: checking if worker %s has joined", p.leader, h.Metadata.Hostname)
-		ready, err := h.KubeNodeReady()
-		if err != nil {
+		if err := node.KubeNodeReadyFunc(h)(context.Background()); err != nil {
 			log.Debugf("%s: failed to get ready status: %s", h, err.Error())
+		} else {
+			h.Metadata.Ready = true
 		}
-		h.Metadata.Ready = ready
 	}
 
 	return nil
@@ -224,17 +231,5 @@ func (p *GatherK0sFacts) needsUpgrade(h *cluster.Host) bool {
 		return true
 	}
 
-	log.Debugf("%s: checking if %s is an upgrade from %s", h, p.Config.Spec.K0s.Version, h.Metadata.K0sRunningVersion)
-	target, err := version.NewVersion(p.Config.Spec.K0s.Version)
-	if err != nil {
-		log.Warnf("%s: failed to parse target version: %s", h, err.Error())
-		return false
-	}
-	current, err := version.NewVersion(h.Metadata.K0sRunningVersion)
-	if err != nil {
-		log.Warnf("%s: failed to parse running version: %s", h, err.Error())
-		return false
-	}
-
-	return target.GreaterThan(current)
+	return p.Config.Spec.K0s.Version.GreaterThan(h.Metadata.K0sRunningVersion)
 }
