@@ -1,11 +1,14 @@
 package phase
 
 import (
+	"context"
 	"fmt"
 	"time"
 
 	"github.com/k0sproject/k0sctl/pkg/apis/k0sctl.k0sproject.io/v1beta1"
 	"github.com/k0sproject/k0sctl/pkg/apis/k0sctl.k0sproject.io/v1beta1/cluster"
+	"github.com/k0sproject/k0sctl/pkg/node"
+	"github.com/k0sproject/k0sctl/pkg/retry"
 	"github.com/k0sproject/rig/exec"
 	log "github.com/sirupsen/logrus"
 )
@@ -27,7 +30,7 @@ func (p *InstallWorkers) Prepare(config *v1beta1.Cluster) error {
 	p.Config = config
 	var workers cluster.Hosts = p.Config.Spec.Hosts.Workers()
 	p.hosts = workers.Filter(func(h *cluster.Host) bool {
-		return !h.Reset && !h.Metadata.NeedsUpgrade && (h.Metadata.K0sRunningVersion == "" || !h.Metadata.Ready)
+		return !h.Reset && !h.Metadata.NeedsUpgrade && (h.Metadata.K0sRunningVersion == nil || !h.Metadata.Ready)
 	})
 	p.leader = p.Config.Spec.K0sLeader()
 
@@ -57,7 +60,7 @@ func (p *InstallWorkers) Run() error {
 
 	err := p.parallelDo(p.hosts, func(h *cluster.Host) error {
 		log.Infof("%s: validating api connection to %s", h, url)
-		if err := h.WaitHTTPStatus(healthz, 200, 401); err != nil {
+		if err := retry.Times(context.Background(), 2, node.HTTPStatusFunc(h, healthz, 200, 401)); err != nil {
 			return fmt.Errorf("failed to connect from worker to kubernetes api at %s - check networking", url)
 		}
 		return nil
@@ -150,7 +153,8 @@ func (p *InstallWorkers) Run() error {
 			log.Debugf("%s: not waiting because --no-wait given", h)
 		} else {
 			log.Infof("%s: waiting for node to become ready", h)
-			if err := h.WaitKubeNodeReady(); err != nil {
+
+			if err := retry.Timeout(context.TODO(), retry.DefaultTimeout, node.KubeNodeReadyFunc(h)); err != nil {
 				return err
 			}
 			h.Metadata.Ready = true
