@@ -1,6 +1,8 @@
 package phase
 
 import (
+	"fmt"
+
 	"github.com/k0sproject/k0sctl/pkg/apis/k0sctl.k0sproject.io/v1beta1"
 	"github.com/k0sproject/k0sctl/pkg/apis/k0sctl.k0sproject.io/v1beta1/cluster"
 	"github.com/logrusorgru/aurora"
@@ -59,6 +61,15 @@ type Manager struct {
 	ConcurrentUploads int
 }
 
+// NewManager creates a new Manager
+func NewManager(config *v1beta1.Cluster) (*Manager, error) {
+	if config == nil {
+		return nil, fmt.Errorf("config is nil")
+	}
+
+	return &Manager{Config: config}, nil
+}
+
 // AddPhase adds a Phase to Manager
 func (m *Manager) AddPhase(p ...phase) {
 	m.phases = append(m.phases, p...)
