@@ -1,13 +1,14 @@
 package phase
 
 import (
+	"context"
 	"fmt"
-	"strings"
+	"time"
 
-	"github.com/Masterminds/semver"
 	"github.com/k0sproject/k0sctl/pkg/apis/k0sctl.k0sproject.io/v1beta1"
 	"github.com/k0sproject/k0sctl/pkg/apis/k0sctl.k0sproject.io/v1beta1/cluster"
-	"github.com/k0sproject/version"
+	"github.com/k0sproject/k0sctl/pkg/node"
+	"github.com/k0sproject/k0sctl/pkg/retry"
 	log "github.com/sirupsen/logrus"
 )
 
@@ -62,25 +63,15 @@ func (p *UpgradeControllers) Run() error {
 			return fmt.Errorf("k0s binary tempfile not found on host")
 		}
 		log.Infof("%s: starting upgrade", h)
-		if p.needsMigration(h) {
-			if err := p.migrateService(h); err != nil {
-				return err
-			}
-		} else {
-			log.Debugf("%s: stop service", h)
-			if err := h.Configurer.StopService(h, h.K0sServiceName()); err != nil {
-				return err
-			}
-		}
-		if err := h.WaitK0sServiceStopped(); err != nil {
+		log.Debugf("%s: stop service", h)
+		if err := h.Configurer.StopService(h, h.K0sServiceName()); err != nil {
 			return err
 		}
-		version, err := version.NewVersion(p.Config.Spec.K0s.Version)
-		if err != nil {
-			return err
+		if err := retry.Timeout(context.TODO(), retry.DefaultTimeout, node.ServiceStoppedFunc(h, h.K0sServiceName())); err != nil {
+			return fmt.Errorf("wait for k0s service stop: %w", err)
 		}
 		log.Debugf("%s: update binary", h)
-		if err := h.UpdateK0sBinary(h.Metadata.K0sBinaryTempFile, version); err != nil {
+		if err := h.UpdateK0sBinary(h.Metadata.K0sBinaryTempFile, p.Config.Spec.K0s.Version); err != nil {
 			return err
 		}
 
@@ -96,53 +87,39 @@ func (p *UpgradeControllers) Run() error {
 			return err
 		}
 		log.Infof("%s: waiting for the k0s service to start", h)
-		if err := h.WaitK0sServiceRunning(); err != nil {
-			return err
+		if err := retry.Timeout(context.TODO(), retry.DefaultTimeout, node.ServiceRunningFunc(h, h.K0sServiceName())); err != nil {
+			return fmt.Errorf("k0s service start: %w", err)
 		}
 		port := 6443
 		if p, ok := p.Config.Spec.K0s.Config.Dig("spec", "api", "port").(int); ok {
 			port = p
 		}
-		if err := h.WaitKubeAPIReady(port); err != nil {
-			return err
+
+		if err := retry.Timeout(context.TODO(), retry.DefaultTimeout, node.KubeAPIReadyFunc(h, port)); err != nil {
+			return fmt.Errorf("kube api did not become ready: %w", err)
 		}
-
-	}
-	return nil
-}
-
-func (p *UpgradeControllers) needsMigration(h *cluster.Host) bool {
-	log.Debugf("%s: checking need for 0.10 --> 0.11 migration", h)
-	c, _ := semver.NewConstraint("< 0.11-0")
-	current, err := semver.NewVersion(h.Metadata.K0sRunningVersion)
-	if err != nil {
-		log.Warnf("%s: failed to parse version info: %s", h, err.Error())
-		return false
 	}
 
-	return c.Check(current)
-}
+	leader := p.Config.Spec.K0sLeader()
+	if NoWait {
+		log.Warnf("%s: skipping scheduler and system pod checks because --no-wait given", leader)
+		return nil
+	}
 
-func (p *UpgradeControllers) migrateService(h *cluster.Host) error {
+	log.Infof("%s: waiting for the scheduler to become ready", leader)
+	if err := retry.Timeout(context.TODO(), retry.DefaultTimeout, node.ScheduledEventsAfterFunc(leader, time.Now())); err != nil {
+		if !Force {
+			return fmt.Errorf("failed to observe scheduling events after api start-up, you can ignore this check by using --force: %w", err)
+		}
+		log.Warnf("%s: failed to observe scheduling events after api start-up: %s", leader, err)
+	}
 
-	log.Infof("%s: updating legacy 'k0sserver' service to '%s'", h, h.K0sServiceName())
-	if err := h.Configurer.StopService(h, "k0sserver"); err != nil {
-		return err
-	}
-	sp, err := h.Configurer.ServiceScriptPath(h, "k0sserver")
-	if err != nil {
-		return err
-	}
-	if sp == "" {
-		return fmt.Errorf("service script path resolved to empty string")
-	}
-	log.Debugf("%s: found old service path: %s", h, sp)
-	newPath := strings.Replace(sp, "k0sserver", h.K0sServiceName(), 1)
-	if err != nil {
-		return err
-	}
-	if err := h.Configurer.MoveFile(h, sp, newPath); err != nil {
-		return err
+	log.Infof("%s: waiting for system pods to become ready", leader)
+	if err := retry.Timeout(context.TODO(), retry.DefaultTimeout, node.SystemPodsRunningFunc(leader)); err != nil {
+		if !Force {
+			return fmt.Errorf("all system pods not running after api start-up, you can ignore this check by using --force: %w", err)
+		}
+		log.Warnf("%s: failed to observe system pods running after api start-up: %s", leader, err)
 	}
 
 	return nil
