@@ -1,13 +1,15 @@
 package phase
 
 import (
+	"context"
 	"fmt"
 	"math"
 
 	"github.com/gammazero/workerpool"
 	"github.com/k0sproject/k0sctl/pkg/apis/k0sctl.k0sproject.io/v1beta1"
 	"github.com/k0sproject/k0sctl/pkg/apis/k0sctl.k0sproject.io/v1beta1/cluster"
-	"github.com/k0sproject/version"
+	"github.com/k0sproject/k0sctl/pkg/node"
+	"github.com/k0sproject/k0sctl/pkg/retry"
 	log "github.com/sirupsen/logrus"
 )
 
@@ -106,15 +108,13 @@ func (p *UpgradeWorkers) upgradeWorker(h *cluster.Host) error {
 	if err := h.Configurer.StopService(h, h.K0sServiceName()); err != nil {
 		return err
 	}
-	if err := h.WaitK0sServiceStopped(); err != nil {
-		return err
-	}
-	version, err := version.NewVersion(p.Config.Spec.K0s.Version)
-	if err != nil {
+
+	if err := retry.Timeout(context.TODO(), retry.DefaultTimeout, node.ServiceStoppedFunc(h, h.K0sServiceName())); err != nil {
 		return err
 	}
+
 	log.Debugf("%s: update binary", h)
-	if err := h.UpdateK0sBinary(h.Metadata.K0sBinaryTempFile, version); err != nil {
+	if err := h.UpdateK0sBinary(h.Metadata.K0sBinaryTempFile, p.Config.Spec.K0s.Version); err != nil {
 		return err
 	}
 
@@ -129,18 +129,19 @@ func (p *UpgradeWorkers) upgradeWorker(h *cluster.Host) error {
 	if err := h.Configurer.StartService(h, h.K0sServiceName()); err != nil {
 		return err
 	}
-	if !p.NoDrain {
-		log.Debugf("%s: marking node schedulable again", h)
-		if err := p.leader.UncordonNode(h); err != nil {
-			return err
-		}
-	}
 	if NoWait {
 		log.Debugf("%s: not waiting because --no-wait given", h)
 	} else {
 		log.Infof("%s: waiting for node to become ready again", h)
-		if err := h.WaitKubeNodeReady(); err != nil {
-			return err
+		if err := retry.Timeout(context.TODO(), retry.DefaultTimeout, node.KubeNodeReadyFunc(h)); err != nil {
+			return fmt.Errorf("node did not become ready: %w", err)
+		}
+
+		if !p.NoDrain {
+			log.Debugf("%s: marking node schedulable again", h)
+			if err := p.leader.UncordonNode(h); err != nil {
+				return fmt.Errorf("uncordon node: %w", err)
+			}
 		}
 		h.Metadata.Ready = true
 	}
