@@ -1,4 +1,4 @@
-#!/bin/bash
+#!/usr/bin/env sh
 
 K0SCTL_CONFIG=${K0SCTL_CONFIG:-"k0sctl.yaml"}
 
@@ -16,13 +16,13 @@ echo "* Starting apply"
 echo "* Apply OK"
 
 echo "* Verify hooks were executed on the host"
-footloose ssh root@manager0 -- grep -q hello apply.hook
+bootloose ssh root@manager0 -- grep -q hello apply.hook
 
 echo "* Verify 'k0sctl kubeconfig' output includes 'data' block"
 ../k0sctl kubeconfig --config k0sctl.yaml | grep -v -- "-data"
 
 echo "* Run kubectl on controller"
-footloose ssh root@manager0 -- k0s kubectl get nodes
+bootloose ssh root@manager0 -- k0s kubectl get nodes
 
 echo "* Downloading kubectl for local test"
 downloadKubectl
@@ -34,9 +34,8 @@ echo "* Using k0sctl kubecofig locally"
 ../k0sctl kubeconfig --config k0sctl.yaml > kubeconfig
 
 echo "* Output:"
-cat kubeconfig | grep -v -- "-data"
+grep -v -- -data kubeconfig
 
 echo "* Running kubectl"
 ./kubectl --kubeconfig kubeconfig get nodes
 echo "* Done"
-
